# chloro - personal space
# Copyright (C) 2019  Frederic Peters
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import html
import re
import urllib.parse

from django.db import models, transaction
from django.utils.html import strip_tags
from django.utils.translation import ugettext_lazy as _
from taggit.managers import TaggableManager

from .fields import RichTextField


class Note(models.Model):
    title = models.CharField(_('Title'), max_length=150)
    slug = models.SlugField(_('Slug'), max_length=150)
    text = RichTextField(_('Text'), blank=True, null=True)
    plain_text = models.TextField(blank=True, null=True)
    tags = TaggableManager(_('Tags'), blank=True)
    published = models.BooleanField(_('Published'), default=True)
    creation_timestamp = models.DateTimeField(auto_now_add=True)
    last_update_timestamp = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ['-creation_timestamp']

    def get_absolute_url(self):
        if self.slug == 'index':
            return '/'
        return '/%s/' % self.slug

    def lang(self):
        if self.tags.filter(name='lang-en').exists():
            return 'en'
        return 'fr'

    def save(self, *args, **kwargs):
        if kwargs.get('update_fields') is None or 'plain_text' in kwargs.get('update_fields'):
            self.plain_text = re.sub(
                r"[’'\"«»,;:\. ]", ' ', html.unescape(strip_tags(self.text)).replace('’', ' ')
            )
        if not kwargs.get('update_fields'):
            self.maintain_interlinks()
        return super().save(*args, **kwargs)

    def maintain_interlinks(self):
        links = re.findall(r'href="(.*?)"', self.text or '')
        with transaction.atomic():
            Interlink.objects.filter(note1=self).delete()
            for link in links:
                parsed = urllib.parse.urlparse(link)
                if parsed.netloc != '':
                    continue
                path_parts = parsed.path.strip('/').split()
                if len(path_parts) != 1:
                    continue
                try:
                    note2 = Note.objects.get(slug=path_parts[0])
                    Interlink.objects.create(note1=self, note2=note2)
                except Note.DoesNotExist:
                    pass


class Interlink(models.Model):
    note1 = models.ForeignKey(Note, on_delete=models.SET_NULL, null=True, related_name='+')
    note2 = models.ForeignKey(Note, on_delete=models.SET_NULL, null=True, related_name='linking_note')
